@@ -1,7 +1,7 @@
 use std::collections::HashMap;
 
 use anyhow::{Error, Result};
-use flexi_logger::{Duplicate, LogTarget, Logger};
+use flexi_logger::{Duplicate, FileSpec, Logger};
 use structopt::StructOpt;
 
 use rest::{netbox, netshot};
@@ -9,7 +9,7 @@ use rest::{netbox, netshot};
 mod common;
 mod rest;
 
-#[derive(Debug, StructOpt)]
+#[derive(Debug, StructOpt, Clone)]
 #[structopt(
     name = "netbox2netshot",
     about = "Synchronization tool between netbox and netshot"
@@ -18,6 +18,9 @@ struct Opt {
     #[structopt(short, long, help = "Enable debug/verbose mode")]
     debug: bool,
 
+    #[structopt(long, help = "The directory to log to", default_value = "logs", env)]
+    log_directory: String,
+
     #[structopt(long, help = "The Netshot API URL", env)]
     netshot_url: String,
 
@@ -80,7 +83,7 @@ struct Opt {
 
 /// Main application entrypoint
 fn main() -> Result<(), Error> {
-    let opt = Opt::from_args();
+    let opt: Opt = Opt::from_args();
     let mut logging_level = "info";
     let mut duplicate_level = Duplicate::Info;
     if opt.debug {
@@ -88,8 +91,8 @@ fn main() -> Result<(), Error> {
         duplicate_level = Duplicate::Debug;
     }
 
-    Logger::with_str(logging_level)
-        .log_target(LogTarget::File)
+    Logger::try_with_str(logging_level)?
+        .log_to_file(FileSpec::default().directory(opt.clone().log_directory))
         .duplicate_to_stdout(duplicate_level)
         .start()
         .unwrap();
@@ -118,8 +121,8 @@ fn main() -> Result<(), Error> {
     log::info!("Getting devices list from Netshot");
     let netshot_devices = netshot_client.get_devices()?;
 
-    log::debug!("Building netshot devices hashmap");
-    let netshot_hashmap: HashMap<_, _> = netshot_devices
+    log::debug!("Building netshot devices simplified inventory");
+    let netshot_simplified_inventory: HashMap<_, _> = netshot_devices
         .into_iter()
         .map(|dev| (dev.management_address.ip, dev.name))
         .collect();
@@ -134,8 +137,8 @@ fn main() -> Result<(), Error> {
         netbox_devices.append(&mut vms);
     }
 
-    log::debug!("Building netbox devices hashmap");
-    let mut netbox_hashmap: HashMap<_, _> = netbox_devices
+    log::debug!("Building netbox devices simplified inventory");
+    let netbox_simplified_devices: HashMap<_, _> = netbox_devices
         .into_iter()
         .filter_map(|device| match device.primary_ip4 {
             Some(x) => Some((
@@ -153,45 +156,70 @@ fn main() -> Result<(), Error> {
         .collect();
 
     log::debug!(
-        "Hashmaps: Netbox({}), Netshot({})",
-        netbox_hashmap.len(),
-        netshot_hashmap.len()
+        "Simplified inventories: Netbox({}), Netshot({})",
+        netbox_simplified_devices.len(),
+        netshot_simplified_inventory.len()
     );
 
-    log::debug!("Comparing HashMaps");
-    let mut missing_devices: Vec<String> = Vec::new();
-    for (ip, hostname) in netbox_hashmap {
-        match netshot_hashmap.get(&ip) {
+    log::debug!("Comparing inventories");
+
+    let mut devices_to_register: Vec<String> = Vec::new();
+    for (ip, hostname) in &netbox_simplified_devices {
+        match netshot_simplified_inventory.get(ip) {
             Some(x) => log::debug!("{}({}) is present on both", x, ip),
             None => {
                 log::debug!("{}({}) missing from Netshot", hostname, ip);
-                missing_devices.push(ip);
+                devices_to_register.push(ip.clone());
             }
         }
     }
 
-    log::info!("Found {} devices missing on Netshot", missing_devices.len());
+    let mut devices_to_disable: Vec<String> = Vec::new();
+    for (ip, hostname) in &netshot_simplified_inventory {
+        match netbox_simplified_devices.get(ip) {
+            Some(x) => log::debug!("{}({}) is present on both", x, ip),
+            None => {
+                log::debug!("{}({}) missing from Netbox", hostname, ip);
+                devices_to_disable.push(ip.clone());
+            }
+        }
+    }
+
+    log::info!(
+        "Found {} devices missing on Netshot, to be added",
+        devices_to_register.len()
+    );
+    log::info!(
+        "Found {} devices missing on Netbox, to be disabled",
+        devices_to_disable.len()
+    );
 
     if !opt.check {
-        for device in missing_devices {
-            let registration = netshot_client.register_device(&device, opt.netshot_domain_id);
+        for device in devices_to_register {
+            let registration = netshot_client.register_device(device, opt.netshot_domain_id);
             if let Err(error) = registration {
                 log::warn!("Registration failure: {}", error);
             }
         }
+
+        for device in devices_to_disable {
+            let registration = netshot_client.disable_device(device);
+            if let Err(error) = registration {
+                log::warn!("Disable failure: {}", error);
+            }
+        }
     }
     Ok(())
 }
 
 #[cfg(test)]
 mod tests {
-    use flexi_logger::{LogTarget, Logger};
+    use flexi_logger::{AdaptiveFormat, Logger};
 
     #[ctor::ctor]
     fn enable_logging() {
-        Logger::with_str("debug")
-            .log_target(LogTarget::StdOut)
-            .start()
-            .unwrap();
+        Logger::try_with_str("debug")
+            .unwrap()
+            .adaptive_format_for_stderr(AdaptiveFormat::Detailed);
     }
 }
